//
//  main.swift
//  SwiftCoroutineMemoryLeakTest
//
//  Created by Alex Belozierov on 13.01.2020.
//  Copyright © 2020 Alex Belozierov. All rights reserved.
//

import SwiftCoroutine
import Foundation

print("Hello, World!")

let mutex = NSLock()
var totalInit = 0
var currentInit = 0

class TestClass {
    
    var a, b, c, d: Int
    
    init(a: Int, b: Int, c: Int, d: Int) {
        self.a = a
        self.b = b
        self.c = c
        self.d = d
        mutex.lock()
        totalInit += 1
        currentInit += 1
        mutex.unlock()
    }
    
    deinit {
        mutex.lock()
        currentInit -= 1
        mutex.unlock()
    }
    
}

let group = DispatchGroup()

func global() {
    for i in 0..<10_000 {
        var localVariable = 0
        let testClass = TestClass(a: i, b: i, c: i, d: i)
        group.enter()
        coroutine(on: .global) {
            localVariable += i
            let coroutine = try Coroutine.current()
            coroutine.suspend(with: coroutine.resume)
            testClass.a += i
            group.leave()
        }
    }
    group.wait()
    
}

func syncStartGlobalResume() {
    for i in 0..<100_000 {
        var localVariable = 0
        let testClass = TestClass(a: i, b: i, c: i, d: i)
        group.enter()
        coroutine(on: .sync) {
            localVariable += i
            try Coroutine.setDispatcher(.global)
            testClass.a += i
            group.leave()
        }
    }
    group.wait()
}

func test() {
    syncStartGlobalResume()
    global()
}

for _ in 0..<50 {
    test()
}

print("test finished")
print("currentInit", currentInit)
print("totalInit", totalInit)

group.enter()
group.wait()
